#!/usr/bin/env python3
# -*- coding: utf-8 -*-
from Commands.PythonCommandBase import PythonCommand, ImageProcPythonCommand
from Commands.Keys import KeyPress, Hat, Button, Direction, Stick
import numpy as np
import enum
from pip._internal import main as _main
import importlib
import unicodedata
import re

DEBUG = True

class KeyBoardTest(ImageProcPythonCommand):
	NAME = 'KeyBoardTest'
	
	def __init__(self, cam):
		super().__init__(cam)
		_import('pykakasi','pykakasi')

	def do(self):		
		#文字列入力画面に遷移
		self.press(Button.X, 0.1, 0.5)
		#不要な文字列を削除する
		self.KeyBoard_Init()#キーボード画面に遷移した場合、1度のみ実行する。
		self.PressKey(KEY.BACKSPACE)		
		self.wait(0.1*6) #0.1sec × (削除したい文字数)が待機時間の目安
		self.ReleaseKey(KEY.BACKSPACE)
		print('キーボードによる文字列入力(Leonardoのみ)\r\n')
		self.TypeString('かわEヌメラ')
		self.finish()

		'''
		#Twitter投稿画面等で試してください
		#文字列送信(改行なし)
		self.TypeString('テスト')
		self.TypeString('かいぎょうなし')
		self.TypeString('()')
		self.TypeKey(KEY.ENTER)
		self.TypeKey(KEY.ENTER)
		
		#文字列送信(改行あり)
		self.TypeStringln('ぬめらカワイイ')
		self.TypeStringln('ぬめらkawaii')
		self.TypeStringln('ヌメラかわいい')
		self.TypeStringln('ヌメラkawaii')
		self.TypeStringln('kawaiiぬめら')
		self.TypeStringln('kawaiiヌメラ')
		self.ResetController()
		'''
		'''
		#レイドパスワード入力(ローカル)
		print(self.TypePassWord_Local('2460'))		#True
		print(self.TypePassWord_Local('2460a'))		#False
		'''
		'''
		#レイドパスワード入力(オンライン)
		print(self.TypePassWord_Online('CVBNM1'))	#True
		print(self.TypePassWord_Online('CVBNMI'))	#False
		self.finish()
		'''
		self.finish()
######################################################################################################
	'''
	公開関数
	'''
	#キーボードの初期化関数(キーボード画面が表示されたときに1度だけ実行すること)
	def KeyBoard_Init(self):
		'''
		キーボードの初期化関数(キーボード画面が表示されたときに1度だけ実行すること)
		'''
		self.KeyBoardMode = 0
		self.KeyBoardMode_buff = 0
		self.keys.ser.writeRow('end')
		self.wait(0.05)

	#キーボード入力
	def TypeString_Roman(self, sendstr):
		#print('入力文字列:' + sendstr)
		sendstr = '"' + sendstr + '"'
		self.keys.ser.writeRow(sendstr)

	#キーボードによる文字列入力(改行なし)
	def TypeString(self, text, conversion=False):
		'''
		キーボードによる文字列入力(改行なし)

		Parameters
		----------
		text : str
		キーボードで入力したい文字列
		----------
		conversion : bool
		キーボードによる変換機能付きかどうか
		'''
		print('入力文字列:' + text)
		text = text.replace("ん","んん")
		text = text.replace("ン","ンン")
		splitlist = self.SplitByCharacterType(text)
		self.debuglog(splitlist)
		Romanlist = self.ConvertToRoman(splitlist)
		self.debuglog(Romanlist)
		for roman_item in Romanlist:
			self.ChangeKeyBoardMode(roman_item[1])
			self.TypeString_Roman(roman_item[0])
			self.wait(0.1*len(roman_item[0]))
			if conversion == True:
				self.TypeKey(KEY.ENTER)

	#キーボードによる文字列入力(改行あり)
	def TypeStringln(self, text, conversion=False):
		'''
		キーボードによる文字列入力(改行あり)
		
		Parameters
		----------
		text : str
		キーボードで入力したい文字列
		----------
		conversion : bool
		キーボードによる変換機能付きかどうか
		'''
		self.TypeString(text, conversion)
		self.TypeKey(KEY.ENTER)
	
	#パスワード入力(ローカルレイド)
	def TypePassWord_Local(self, password):
		'''
		パスワード入力(ローカルレイド)
		
		Parameters
		----------
		password : str
		パスワード(0-9の文字列)
		'''
		status = re.search('^[0-9]+$', password)
		if(status == None):
			print('TypePassWord_Local:不正な引数です。(' + password +')')
			return False
		print('パスワード:' + password)
		self.TypeString_Roman(password)
		self.wait(0.1*len(password))
		return True

	#パスワード入力(オンラインレイド)
	def TypePassWord_Online(self, password):
		'''
		パスワード入力(ローカルレイド)
		
		Parameters
		----------
		password : str
		パスワード(「0-9QWERTYUPASDFGHJKLXCVBNM」の文字列)
		'''
		pw = password.lower()
		status = re.search('^[0-9qwertyupasdfghjklxcvbnm]+$', pw)
		if(status == None):
			print('TypePassWord_Online:不正な引数です。(' + password +')')
			return False
		print('パスワード:' + password)
		self.TypeString_Roman(pw)
		self.wait(0.1*len(pw))
		return True

	#キー入力
	def TypeKey(self, keyvalue, show_value=False):
		'''
		キー入力

		Parameters
		----------
		keyvalue : int
		キーボードで入力したいキー
		----------
		show_value : bool
		入力したキーの値をprintするか
		'''		
		if show_value:
			print(KEY(keyvalue).name)
		sendstr = 'Key ' + str(int(keyvalue))
		self.keys.ser.writeRow(sendstr)

	#キーを押しっぱなしにする
	def PressKey(self, keyvalue, show_value=False):
		'''
		キーを押しっぱなしにする

		Parameters
		----------
		keyvalue : int
		押しっぱなしにしたいキー
		----------
		show_value : bool
		入力したキーの値をprintするか
		'''		
		if show_value:
			print(KEY(keyvalue).name)
		sendstr = 'Press ' + str(int(keyvalue))
		self.keys.ser.writeRow(sendstr)

	#押しっぱなしにしているキーを離す
	def ReleaseKey(self, keyvalue, show_value=False):
		'''
		押しっぱなしにしているキーを離す

		Parameters
		----------
		keyvalue : int
		離したいキー
		----------
		show_value : bool
		入力したキーの値をprintするか
		'''	
		if show_value:
			print(KEY(keyvalue).name)
		sendstr = 'Release ' + str(int(keyvalue))
		self.keys.ser.writeRow(sendstr)

	#入力したパスワードをリセットする
	def ResetPassword(self):
		self.PressKey(KEY.BACKSPACE)		
		self.wait(0.1*6) #0.1sec × (削除したい文字数)が待機時間の目安
		self.ReleaseKey(KEY.BACKSPACE)

	#コントローラーを無入力状態に戻す
	def ResetController(self):
		self.keys.ser.writeRow('3 8 80 80 80 80')

	#デバッグ用のlogを残す関数
	def debuglog(self, text):
		if DEBUG:
			print(text)

######################################################################################################
	'''
	キーボード系
	'''
	#文字種に応じて文字列を分割する
	def SplitByCharacterType(self, text):
		txt = text
		resultlist = []
		patternlist = [
			#【参考】文字コード:https://0g0.org/category/3040-309F/1/
			'^[\u3040-\u309F]+', # ひらがな u+3040 - u+309F
			'^[\u30A0-\u30FB\u30FD-\u30FF]+', # カタカナ u+30A0 - u+30FF(【除】「ー」:u+30FC)
			#'^[\u30A0-\u30FF]+', # カタカナ u+30A0 - u+30FF
			'^[a-zA-Z0-9\-!@#\$%\^&\*\(\)_~`=\\\+\{\}\|\[\]<>;:"\',\.\?/]+',	 # 英数字・記号
			'^[ー～「」、。・]+'#全角記号
		]
		i = 0
		while i < len(patternlist):
			status = re.search(patternlist[i], txt)
			if(status != None):
				txt = txt[status.span()[1]:]
				strlist_tmp = [status.group(), i]
				resultlist.append(strlist_tmp)
				i = 0
				continue
			i = i + 1
		return resultlist

	#ローマ字に変換する
	def ConvertToRoman(self, splitlist):
		romanlist = []
		for splititem in splitlist:
			romanlist_tmp = [self.ConvertToRoman_sub(splititem[0],splititem[1]),splititem[1]]
			romanlist.append(romanlist_tmp)
		return romanlist

	#ローマ字に変換する
	def ConvertToRoman_sub(self, text, mode):
		kks = pykakasi.kakasi()
		if mode == KEYBOARD_MODE.HIRAGANA: 
			kks.setMode("H","a") # default: Hiragana -> Roman
		elif mode == KEYBOARD_MODE.KATAKANA: 
			kks.setMode("K","a") # default: Katakana -> Roman
		elif mode == KEYBOARD_MODE.DOUBLEBYTESYMBOL: 
			txt = ''
			DoubleByteSymbolConvertList = {
				'ー':'-',
				'～':'~',
				'「':'[',
				'」':']',
				'、':',',
				'。':'.',
				'・':'/'
			}
			for i in range(len(text)):
				txt = txt + DoubleByteSymbolConvertList[text[i]]
			return txt
		else:
			return text
		return kks.convert(text)[0]['kunrei']#hepburn/kunrei/passport

	def ChangeKeyBoardMode(self, mode):
		if (self.KeyBoardMode == KEYBOARD_MODE.HIRAGANA) and (mode == KEYBOARD_MODE.KATAKANA):
			self.TypeKey(KEY.JP_HIRAGANA)
		elif (self.KeyBoardMode == KEYBOARD_MODE.HIRAGANA) and (mode == KEYBOARD_MODE.ALPHANUMERIC_SYMBOL):
			self.TypeKey(KEY.JP_HANZEN)
		elif (self.KeyBoardMode == KEYBOARD_MODE.KATAKANA) and (mode == KEYBOARD_MODE.HIRAGANA):
			self.TypeKey(KEY.JP_HIRAGANA)
		elif (self.KeyBoardMode == KEYBOARD_MODE.KATAKANA) and (mode == KEYBOARD_MODE.ALPHANUMERIC_SYMBOL):
			self.TypeKey(KEY.JP_HIRAGANA)
			self.TypeKey(KEY.JP_HANZEN)
		elif (self.KeyBoardMode == KEYBOARD_MODE.ALPHANUMERIC_SYMBOL) and (mode == KEYBOARD_MODE.HIRAGANA):
			self.TypeKey(KEY.JP_HANZEN)
		elif (self.KeyBoardMode == KEYBOARD_MODE.ALPHANUMERIC_SYMBOL) and (mode == KEYBOARD_MODE.KATAKANA):
			self.TypeKey(KEY.JP_HANZEN)
			self.TypeKey(KEY.JP_HIRAGANA)
		elif (self.KeyBoardMode == KEYBOARD_MODE.ALPHANUMERIC_SYMBOL) and (mode == KEYBOARD_MODE.DOUBLEBYTESYMBOL):
			self.TypeKey(KEY.JP_HANZEN)
			self.KeyBoardMode = KEYBOARD_MODE.HIRAGANA
			return
		else:
			return		
		self.KeyBoardMode = mode
		self.KeyBoardMode_buff = mode

######################################################################################################
#キーボードのかな/カナ/英字を管理するためのクラス
class KEYBOARD_MODE(enum.IntEnum):
	HIRAGANA			= 0 #ひらがな
	KATAKANA 				= 1 #カタカナ
	ALPHANUMERIC_SYMBOL		= 2 #英数字
	DOUBLEBYTESYMBOL		= 3 #全角記号

#キーコードを管理するためのクラス
class KEY(enum.IntEnum):
	SPACE			= 32
	ENTER			= 40
	BACKSPACE		= 42
	DELETE			= 76
	UP_ARROW		= 79
	DOWN_ARROW		= 80
	LEFT_ARROW		= 81
	RIGHT_ARROW		= 82
	JP_HANZEN		= 0x35
	JP_BACKSLASH	= 0x87
	JP_HIRAGANA 	= 0x88
	#JP_YEN			= 0x89	#「￥」:Switchでは非対応？
	JP_HENKAN		= 0x8A
	#JP_MUHENKAN	= 0x8B	#無変換:Switchでは非対応？	
######################################################################################################
'''
import、なければインストール
'''
def _import(name, module, ver=None):
    try:
        globals()[name] = importlib.import_module(module)
    except ImportError:
        try:
            if ver is None:
                _main(['install', module])
            else:
                _main(['install', '{}=={}'.format(module, ver)])
            globals()[name] = importlib.import_module(module)
        except:
            print("can't import: {}".format(module))
######################################################################################################

